/*
 * Copyright (c) 2013,2018 CNRS/LAAS
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
#ifndef _DAQFLEX_INT_H
#define _DAQFLEX_INT_H

#include <libusb.h>

/* USB Vendor/product IDs */
#define MCC_VENDOR_ID 0x09db

#define MCC_PRODUCT_USB_1608_FS_PLUS 0x00ea
#define MCC_PRODUCT_USB_7204 0x00f0
#define MCC_PRODUCT_USB_7202 0x00f2
#define MCC_PRODUCT_USB_2001_TC 0x00f9
#define MCC_PRODUCT_USB_1608_GX 0x0111
#define MCC_PRODUCT_USB_1608_GX_2A0 0x0112
#define MCC_PRODUCT_USB_201 0x0113

/* timeout for USB transfers */
#define DAQFLEX_USB_TIMEOUT 1000 /* ms */

/* Frame type for ASCII commands */
#define STRINGMESSAGE 0x80

/* Maximum DAQFlex Message length */
#define MAX_MESSAGE_LENGTH 64

/* Intput volt scale */
#define DAQFLEX_USB201_FULLSCALE 20.0 /* -10..+10 V */
#define DAQFLEX_USB201_MIN_VOLTAGE -10.0

/* Maximum number of channels for USB-201 devices*/
#define DAQFLEX_USB201_MAX_NUM_CHANS 8


/*
 * mcc daqflex descriptor
 *
 * XXX more or less specific to USB-201 devices. should be split up
 * in generic daqflex part and device dependent part
 */
struct mccdevice {
	/* libusb specific */
	libusb_device_handle *usbDev;
	libusb_device **devices;
	uint8_t inEndPoint;
	uint8_t outEndPoint;
	uint16_t bulkPacketSize;
	/* USB-201 specific part */
	uint16_t resolution;	/* mask of active bits in data values  */
	int lowChannel;		/* lowest channel to read */
	int highChannel;	/* highest channel to read */
	int numChans;		/* number of selected channels */
	int bufSize;		/* buffer size (in 16 bit values) */
	int16_t *buffer;	/* buffer */
	int readPtr;		/* read pointer */
	int writePtr;		/* write pointer */
	int numValues;		/* number of values in buffer */
	double slope[DAQFLEX_USB201_MAX_NUM_CHANS]; /* calibration */
	double offset[DAQFLEX_USB201_MAX_NUM_CHANS]; /* data */
};

#endif
